/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.config;

import java.io.File;

import rasmus_torkel.config.ConfigProcessor;
import rasmus_torkel.config.PropertySourceDescription;
import rasmus_torkel.config.exception.ConfigSyntaxException;
import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;

public class ConfigProcessorTest extends UnitTest
{
    private final File _dummyFile = new File("dummyFile.txt");
    private final TestConfigProcessor _processor = new TestConfigProcessor();
    
    private int _lineNr = 0;
    
    private class TestConfigProcessor extends ConfigProcessor
    {
        public
        TestConfigProcessor()
        {
            super(new PropertySourceDescription("unit testing code"));
        }
        public void
        testOneLine(String  line,
                    String  name,
                    boolean expectSuccess,
                    String  expectedValue)
        {
            printTestPartSeparator();
            System.out.println("Doing line: " + line);
            _lineNr++;
            try
            {
                tryParam(line, _dummyFile, _lineNr);
                String value = _processor.getValue(name);
                System.out.println(name + " = " + value);
                if (!expectSuccess)
                {
                    System.out.println("Expected success");
                    throw new UnitTestError(ConfigProcessorTest.class);
                }
                if (   value == null && expectedValue != null
                    || value != null && !value.equals(expectedValue))
                {
                    System.out.println("Expected " + expectedValue);
                    throw new UnitTestError(ConfigProcessorTest.class);
                }
            }
            catch (ConfigSyntaxException e)
            {
                System.out.println("  " + e);
                if (expectSuccess)
                {
                    System.out.println("Expected success");
                    throw new UnitTestError(ConfigProcessorTest.class);
                }
            }
            catch (RuntimeException e)
            {
                System.out.println("Unexpected and non-standard throwable: " + e);
                throw new UnitTestError(ConfigProcessorTest.class);
            }
        }
    }
    
    public void
    testOneLine(String  line,
                String  name,
                boolean expectSuccess,
                String  expectedValue)
    {
        _processor.testOneLine(line, name, expectSuccess, expectedValue);
    }
    
    public void
    testOneLine(String  line,
                String  name,
                String  expectedValue)
    {
        testOneLine(line, name, true, expectedValue);
    }
    
    public void
    testBadLine(String  line,
                String  name)
    {
        testOneLine(line, name, false, null);
    }
    
    public void
    printAll()
    {
        printTestPartSeparator();
        System.out.println("Printing config string");
        System.out.println(_processor.makeConfigString());
        System.out.println("Printing trace");
        System.out.println(_processor.makeTrace());
        System.out.println("Printing annotated config string");
        System.out.println(_processor.makeAnnotatedConfigString());
        System.out.println("Printing properties view");
        System.out.println(_processor.makePropertiesView());
    }
    
    public static void
    main(String[] args)
    {
        ConfigProcessorTest t = new ConfigProcessorTest();
        t.testOneLine("v1 = aaa", "v1", "aaa");
        t.testOneLine("v2 = $v1", "v2", "aaa");
        t.testOneLine("v1  =  bbb ", "v1", "bbb");
        t.testOneLine("v3=ccc ", "v3", "ccc");
        t.testOneLine(" v = x ", "v", "x");
        t.testOneLine("w=y", "w", "y");
        t.testOneLine("empty1 = ", "empty1", "");
        t.testOneLine("empty2=", "empty1", "");
        t.testOneLine("v3=CCC", "v3", "CCC");
        t.testOneLine("$v2=$v", "aaa", "x");
        t.testOneLine("e1=e2=e3", "e1", "e2=e3");
        t.testOneLine("space=\" \"", "space", " ");
        t.testOneLine("quote=\"", "quote", "\"");
        t.testOneLine("new line=new\\r\\nline", "new line", "new\r\nline");
        t.testOneLine("firstName=Rasmus", "firstName", "Rasmus");
        t.testOneLine("lastName=Torkel", "lastName", "Torkel");
        t.testOneLine("name=${firstName}${space}${lastName}", "name", "Rasmus Torkel");
        t.testOneLine("\"name\"=\"${firstName}${space}${lastName}\"", "name", "Rasmus Torkel");
        t.testOneLine("qString=\"quote at start", "qString", "\"quote at start");
        t.testOneLine("qString=quote at end\"", "qString", "quote at end\"");
        t.testOneLine("qString=\" space at start\"", "qString", " space at start");
        t.testOneLine("qString=\"space at end \"", "qString", "space at end ");
        t.testOneLine("qString=\"\"keeping quotes\"\"", "qString", "\"keeping quotes\"");
        System.setProperty("firstName", "Dora");
        System.setProperty("lastName", "Both");
        t.testOneLine("firstName=$$firstName", "firstName", "Dora");
        t.testOneLine("name=$${firstName}${space}$${lastName}", "name", "Dora Both");
        t.testOneLine("name=$${firstName}${space}$$lastName", "name", "Dora Both");
        t.testOneLine("baseDir=c:/runtime", "baseDir", "c:/runtime");
        t.testOneLine("configDir=${baseDir}/config", "configDir", "c:/runtime/config");
        t.testOneLine("configFile=${configDir}/app.cfg", "configFile", "c:/runtime/config/app.cfg");
        t.testOneLine("map[1]{aa}=xx", "map[1]{aa}", "xx");
        t.testOneLine("mapx=${map[1]{aa}}+${map[1]{aa}}", "mapx", "xx+xx");
        t.testBadLine("xx=x\\x", "xx");
        t.printAll();
    }
}
