/*
 * Copyright Rasmus Torkel 2012
 * Author    Rasmus Torkel 2012
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.read.impl;

import rasmus_torkel.text.read.exception.DuplicateNodeException;
import rasmus_torkel.text.read.exception.MissingNodeException;
import rasmus_torkel.text.read.exception.SurplusNodeException;
import rasmus_torkel.text.read.exception.UnorderedRetrievalException;
import rasmus_torkel.text.read.exception.WrongNodeException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.exception.XmlNotOnlyTextException;
import rasmus_torkel.xml_basic.read.exception.XmlSyntaxException;
import rasmus_torkel.xml_basic.read.exception.XmlTextAndChildException;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;

public class XmlReaderSuiteNested extends XmlReaderSuite
{
    public static final String N_PERSON = "person";
    public static final String N_NAME = "name";
    public static final String N_FIRST_NAME = "firstName";
    public static final String N_SURNAME = "surname";
    public static final String N_OCCUPATION = "occupation";
    
    public static final String V_FIRST_NAME = "Johann";
    public static final String V_SURNAME = "Torkel";
    public static final String V_OCCUPATION = "Accountant";
    public static final String V_ODD_CHARS = "<&'\">X";
    
    public static final Person PERSON_STANDARD = new Person(V_FIRST_NAME, V_SURNAME, V_OCCUPATION);
    public static final Person PERSON_NO_FIRST_NAME = new Person(null, V_SURNAME, V_OCCUPATION);
    public static final Person PERSON_NO_SURNAME = new Person(V_FIRST_NAME, null, V_OCCUPATION);
    public static final Person PERSON_NO_OCCUPATION = new Person(V_FIRST_NAME, V_SURNAME, null);
    public static final Person PERSON_NO_NAME = new Person(null, null, V_OCCUPATION);
    public static final Person PERSON_NO_CONTENTS = new Person(null, null, null);
    public static final Person PERSON_EMPTY_CONTENTS = new Person("", "", "");
    public static final Person PERSON_ODD_CHARS = new Person(V_ODD_CHARS, V_SURNAME, V_OCCUPATION);
    
    public static final String PERSON_STANDARD_XML1_OK =
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>";
    
    public static final String[] PERSON_STANDARD_XML1_OK_SET =
    {
        PERSON_STANDARD_XML1_OK,
        "<person><name><firstName><![CDATA[Johann]]></firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Joha&#110;&#X6E;</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Joha&#000110;&#x006e;</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<?xxxx?><person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<!DOCTYPE person SYSTEM \"xx.dtd\"><person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<?xxxx?><!DOCTYPE person SYSTEM \"xx.dtd\"><person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><!-- Standard person with comment --><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person>\n" +
        "    <name>\n" +
        "        <firstName>Johann</firstName>\n" +
        "        <surname>Torkel</surname>\n" +
        "    </name>\n" +
        "    <occupation>Accountant</occupation>\n" +
        "</person>",
    };
    
    private static final String[] PERSON_STANDARD_XML1_NON_WS_SET =
    {
        "<person>x<name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name>x<firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName>x<surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname>x</name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name>x<occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation>x</person>",
        "<person>\n" +
        "    <name>\n" +
        "        <firstName>Johann</firstName>\n" +
        "        x\n" +
        "        <surname>Torkel</surname>\n" +
        "    </name>\n" +
        "    <occupation>Accountant</occupation>\n" +
        "</person>",
    };
    
    private static final String PERSON_STANDARD_XML1_CHILD_INSTEAD_OF_TEXT =
        "<person><name><firstName><Johann/></firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>";
    
    private static final String PERSON_BAD_NS_REF =
        "<person>\n" +
        "  <x:name xmlns:x=\"NS1\">\n" +
        "    <x:firstName>Johann</x:firstName>\n" +
        "    <x:surname>Torkel</x:surname>\n" +
        "  </x:name>\n" +
        "  <x:occupation>Accountant</x:occupation>\n" +
        "</person>";
    
    private static final String PERSON_2_NS_1_PREFIX_REF =
        "<person>\n" +
        "  <name xmlns:x=\"NS1\" xmlns:x=\"NS2\">\n" +
        "    <firstName>Johann</firstName>\n" +
        "    <surname>Torkel</surname>\n" +
        "  </name>\n" +
        "  <occupation>Accountant</occupation>\n" +
        "</person>";
    
    private static final String[] PERSON_STANDARD_XML1_BAD_SYNTAX_SET =
    {
        "x<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<? whatever ?>x<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>x",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person><x>x</x>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></personx>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></namex><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstNamex><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surnamex></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupationx></person>",
        "",
        " ",
        "<!-- No content -->",
        "<? whatever ?>",
        "<? whatever ?>\n<!-- No content -->",
        "<?",
        "<?a",
        "<?ab",
        "<?abcde",
    };
    
    private static final String PERSON_STANDARD_XML1_MISSING_FIRST_NAME =
        "<person><name><surname>Torkel</surname></name><occupation>Accountant</occupation></person>";
    
    private static final String PERSON_STANDARD_XML1_MISSING_SURNAME =
        "<person><name><firstName>Johann</firstName></name><occupation>Accountant</occupation></person>";
    
    private static final String PERSON_STANDARD_XML1_MISSING_OCCUPATION =
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name></person>";
    
    private static final String PERSON_STANDARD_XML1_MISSING_NAME =
        "<person><occupation>Accountant</occupation></person>";
    
    private static final String PERSON_STANDARD_XML1_MISSING_CONTENTS =
        "<person></person>";
    
    private static final String PERSON_STANDARD_XML1_MISSING_FIELDS =
        "<person><name></name></person>";
    
    private static final String PERSON_STANDARD_XML1_MISSING_NAME_FIELDS =
        "<person><name></name><occupation>Accountant</occupation></person>";
    
    private static final String[] PERSON_STANDARD_XML1_MISSING_FIELD_SET =
    {
        PERSON_STANDARD_XML1_MISSING_FIRST_NAME,
        PERSON_STANDARD_XML1_MISSING_SURNAME,
        PERSON_STANDARD_XML1_MISSING_OCCUPATION,
        PERSON_STANDARD_XML1_MISSING_FIELDS,
        PERSON_STANDARD_XML1_MISSING_NAME_FIELDS,
    };
    
    private static final String[] PERSON_STANDARD_XML1_MISSING_NODE_SET =
    {
        PERSON_STANDARD_XML1_MISSING_SURNAME,
        PERSON_STANDARD_XML1_MISSING_OCCUPATION,
        PERSON_STANDARD_XML1_MISSING_CONTENTS,
    };
    
    private static final String[] PERSON_STANDARD_XML1_WRONG_NODE_SET =
    {
        PERSON_STANDARD_XML1_MISSING_NAME,
        PERSON_STANDARD_XML1_MISSING_FIRST_NAME,
    };
    
    private static final String PERSON_STANDARD_DUPLICATE_NODE =
        "<person>\n" +
        "    <name><firstName>Johann</firstName><surname>Torkel</surname></name>\n" +
        "    <occupation>Accountant</occupation>\n" +
        "    <occupation>Accountant</occupation>\n" +
        "</person>";
    
    private static final String[] PERSON_STANDARD_XML1_SURPLUS_NODE_SET =
    {
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname><x>xx</x></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation><x/></person>",
    };
    
    private static final String[] PERSON_ODD_CHARS_XML1_OK_SET =
    {
        "<person><name><firstName><![CDATA[<&'\">X]]></firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
        "<person><name><firstName>&lt;&amp;&apos;&quot;&gt;X</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
    };
    
    private static final String PERSON_STANDARD_NS1_OK =
        "<x:person xmlns:x=\"NS1\"><x:name><x:firstName>Johann</x:firstName><x:surname>Torkel</x:surname></x:name><x:occupation>Accountant</x:occupation></x:person>";
    
    private static final String PERSON_STANDARD_NS1_DEFAULT_OK =
        "<person xmlns=\"NS1\"><name><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>";
    
    private static final String PERSON_STANDARD_NS1_PREFIX_TRICKS_OK =
        "<x:person xmlns:x=\"NS1\" xmlns:y=\"NS2\"><y:name xmlns:y=\"NS1\" xmlns:x=\"NS2\"><y:firstName>Johann</y:firstName><y:surname>Torkel</y:surname></y:name><x:occupation>Accountant</x:occupation></x:person>";
    
    private static final String PERSON_STANDARD_NS2_OK =
        "<x:person xmlns:x=\"NS2\"><x:name><x:firstName>Johann</x:firstName><x:surname>Torkel</x:surname></x:name><x:occupation>Accountant</x:occupation></x:person>";
    
    private static final String PERSON_STANDARD_ATTR1_OK =
        "<person firstName=\"Johann\" surname=\"Torkel\" occupation=\"Accountant\"/>";
    
    private static final String[] PERSON_STANDARD_ATTR1_OK_SET =
    {
        PERSON_STANDARD_ATTR1_OK,
        "<person firstName=\"Joha&#110;&#X6E;\" surname=\"Torkel\" occupation=\"Accountant\"/>",
        "<person firstName=\'Johann\' surname=\'Torkel\' occupation=\'Accountant\'/>"
    };
    
    // "<person><name><firstName>&lt;&amp;&apos;&quot;&gt;X</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></person>",
    private static final String[] PERSON_STANDARD_ATTR1_ODD_CHARS_SET =
    {
        "<person firstName=\"&lt;&amp;'&quot;&gt;X\" surname=\"Torkel\" occupation=\"Accountant\"/>",
        "<person firstName=\'&lt;&amp;&apos;\"&gt;X\' surname=\'Torkel\' occupation=\'Accountant\'/>"
    };
    
    private static final String PERSON_STANDARD_TEXT1_OK =
        "<person>Johann<sep/>Torkel<sep/>Accountant</person>";
    
    private static final String PERSON_EMPTY_TEXT1 =
        "<person><sep/><sep/></person>";
    
    private static final String PERSON_STANDARD_REVERSE =
        "<person><occupation>Accountant</occupation><name><surname>Torkel</surname><firstName>Johann</firstName></name></person>";
    
    private static final String PERSON_SEMI_REVERSE =
        "<person><name><surname>Torkel</surname><firstName>Johann</firstName></name><occupation>Accountant</occupation></person>";
    
    private static final String[] PERSON_STANDARD_REVERSE_SURPLUS_NODE_SET =
    {
        "<person><x/><occupation>Accountant</occupation><name><surname>Torkel</surname><firstName>Johann</firstName></name></person>",
        "<person><occupation>Accountant</occupation><x/><name><surname>Torkel</surname><firstName>Johann</firstName></name></person>",
        "<person><occupation>Accountant</occupation><name><x/><surname>Torkel</surname><firstName>Johann</firstName></name></person>",
        "<person><occupation>Accountant</occupation><name><surname>Torkel</surname><x/><firstName>Johann</firstName></name></person>",
        "<person><occupation>Accountant</occupation><name><surname>Torkel</surname><firstName>Johann</firstName><x/></name></person>",
        "<person><occupation>Accountant</occupation><name><surname>Torkel</surname><firstName>Johann</firstName></name><x/></person>",
    };
    
    public static class Person
    {
        public final String _firstName;
        public final String _surname;
        public final String _occupation;
        public final String _string;
        
        public
        Person(String firstName,
               String surname,
               String occupation)
        {
            _firstName = firstName;
            _surname = surname;
            _occupation = occupation;
            _string = N_FIRST_NAME + " = " + _firstName + ", " +
                      N_SURNAME + " = " + _surname + ", " +
                      N_OCCUPATION + " = " + _occupation;
        }
        
        @Override
        public String
        toString()
        {
            return _string;
        }
        
        @Override
        public boolean
        equals(Object other)
        {
            return ((Person)other)._string.equals(_string);
        }
    }
    
    private static final XmlSubTest XML1_STRICT = new XmlSubTest()
    {
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            TagNode nameNode = rootNode.nextChildE(N_NAME);
            String firstName = nameNode.nextTextFieldE(N_FIRST_NAME);
            String surname = nameNode.nextTextFieldE(N_SURNAME);
            nameNode.verifyNoMoreChildren();
            String occupation = rootNode.nextTextFieldE(N_OCCUPATION);
            rootNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "strict1";
        }
    };
    
    private static final XmlSubTest XML_NS1_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns1 = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(N_PERSON, ns1);
            TagNode nameNode = rootNode.nextChildE(N_NAME, ns1);
            String firstName = nameNode.nextTextFieldE(N_FIRST_NAME, ns1);
            String surname = nameNode.nextTextFieldE(N_SURNAME, ns1);
            nameNode.verifyNoMoreChildren();
            String occupation = rootNode.nextTextFieldE(N_OCCUPATION, ns1);
            rootNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "strict1 with namespace";
        }
    };
    
    private static final XmlSubTest XML_ANY_NS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON, XmlNameSpace.ANY);
            TagNode nameNode = rootNode.nextChildE(N_NAME, XmlNameSpace.ANY);
            String firstName = nameNode.nextTextFieldE(N_FIRST_NAME, XmlNameSpace.ANY);
            String surname = nameNode.nextTextFieldE(N_SURNAME, XmlNameSpace.ANY);
            nameNode.verifyNoMoreChildren();
            String occupation = rootNode.nextTextFieldE(N_OCCUPATION, XmlNameSpace.ANY);
            rootNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "any namespace";
        }
    };
    
    private static final XmlSubTest XML_NS1_STRICT_NODE_ID = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns1 = new XmlNameSpace("y", "NS1");
            TagNodeId rootId = new TagNodeId(N_PERSON, ns1);
            TagNodeId nameId = new TagNodeId(N_NAME, ns1);
            TagNodeId firstNameId = new TagNodeId(N_FIRST_NAME, ns1);
            TagNodeId surnameId = new TagNodeId(N_SURNAME, ns1);
            TagNodeId occupationId = new TagNodeId(N_OCCUPATION, ns1);
            
            TagNode rootNode = xmlReader.getRootNode(rootId);
            TagNode nameNode = rootNode.nextChildE(nameId);
            String firstName = nameNode.nextTextFieldE(firstNameId);
            String surname = nameNode.nextTextFieldE(surnameId);
            nameNode.verifyNoMoreChildren();
            String occupation = rootNode.nextTextFieldE(occupationId);
            rootNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
    
        @Override
        public String
        toString()
        {
            return "strict1 with node id";
        }
    };
    
    private static final XmlSubTest XML1_STRICT_WIDTH_FIRST = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            TagNode nameNode = rootNode.nextChildE(N_NAME);
            String occupation = rootNode.nextTextFieldE(N_OCCUPATION);
            rootNode.verifyNoMoreChildren();
            String firstName = nameNode.nextTextFieldE(N_FIRST_NAME);
            String surname = nameNode.nextTextFieldE(N_SURNAME);
            nameNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "strict1, width first";
        }
    };
    
    private static final XmlSubTest XML1_USE_DEFAULTS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            TagNode nameNode = rootNode.nextChildE(N_NAME);
            String firstName = nameNode.nextTextFieldD(N_FIRST_NAME, V_FIRST_NAME);
            String surname = nameNode.nextTextFieldD(N_SURNAME, V_SURNAME);
            nameNode.verifyNoMoreChildren();
            String occupation = rootNode.nextTextFieldD(N_OCCUPATION, V_OCCUPATION);
            rootNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "use defaults";
        }
    };
    
    private static final XmlSubTest XML1_ALLOW_NULLS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            TagNode nameNode = rootNode.nextChildN(N_NAME);
            String firstName = null;
            String surname = null;
            if (nameNode != null)
            {
                firstName = nameNode.nextTextFieldN(N_FIRST_NAME);
                surname = nameNode.nextTextFieldN(N_SURNAME);
                nameNode.verifyNoMoreChildren();
            }
            String occupation = rootNode.nextTextFieldN(N_OCCUPATION);
            rootNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "allow nulls 1";
        }
    };
    
    private static final XmlSubTest XML_ATTR1_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            rootNode.verifyNoMoreChildren();
            String firstName = rootNode.attributeValueE(N_FIRST_NAME);
            String surname = rootNode.attributeValueE(N_SURNAME);
            String occupation = rootNode.attributeValueE(N_OCCUPATION);
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "attr1 strict";
        }
    };
    
    private static final XmlSubTest XML_TEXT1_IGNORE_SEPARATORS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            String firstName = rootNode.nextTextE();
            String surname = rootNode.nextTextE();
            String occupation = rootNode.nextTextE();
            rootNode.verifyNoMoreText();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "text1, ignore separators";
        }
    };
    
    private static final XmlSubTest XML_TEXT1_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            String firstName = rootNode.nextTextE();
            TagNode sep1 = rootNode.nextChildE("sep");
            sep1.verifyNoMoreChildren();
            sep1.verifyNoMoreText();
            String surname = rootNode.nextTextE();
            TagNode sep2 = rootNode.nextChildE("sep");
            sep2.verifyNoMoreText();
            sep2.verifyNoMoreChildren();
            String occupation = rootNode.nextTextE();
            rootNode.verifyNoMoreText();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "text1, strict";
        }
    };
    
    private static final XmlSubTest XML_ANY_ORDER_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            TagNode nameNode = rootNode.anyOrderChildE(N_NAME);
            String firstName = nameNode.anyOrderTextFieldE(N_FIRST_NAME);
            String surname = nameNode.anyOrderTextFieldE(N_SURNAME);
            String occupation = rootNode.anyOrderTextFieldE(N_OCCUPATION);
            rootNode.verifyNoOtherChildren(N_NAME, N_OCCUPATION);
            nameNode.verifyNoOtherChildren(N_FIRST_NAME, N_SURNAME);
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, strict";
        }
    };
    
    private static final XmlSubTest XML_SEMI_ORDERED_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            rootNode.setOrderedRetrieval();
            TagNode nameNode = rootNode.nextChildE(N_NAME);
            String firstName = nameNode.anyOrderTextFieldE(N_FIRST_NAME);
            String surname = nameNode.anyOrderTextFieldE(N_SURNAME);
            String occupation = rootNode.nextTextFieldE(N_OCCUPATION);
            nameNode.verifyNoOtherChildren(N_FIRST_NAME, N_SURNAME);
            rootNode.verifyNoMoreChildren();
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "semi-ordered, strict";
        }
    };
    
    private static final XmlSubTest XML_ANY_ORDER_WITH_DEFAULTS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_PERSON);
            TagNode nameNode = rootNode.anyOrderChildE(N_NAME);
            String firstName = nameNode.anyOrderTextFieldD(N_FIRST_NAME, V_FIRST_NAME);
            String surname = nameNode.anyOrderTextFieldD(N_SURNAME, V_SURNAME);
            String occupation = rootNode.anyOrderTextFieldD(N_OCCUPATION, V_OCCUPATION);
            rootNode.verifyNoOtherChildren(N_NAME, N_OCCUPATION);
            nameNode.verifyNoOtherChildren(N_FIRST_NAME, N_SURNAME);
            
            Person person = new Person(firstName, surname, occupation);
            printTestPartSeparator();
            return person;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, with defaults";
        }
    };
    
    public void
    test()
    {
        announceTestClassStart();
        
        XML1_STRICT.testOk(PERSON_STANDARD_XML1_OK_SET, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML1_STRICT.testOk(PERSON_STANDARD_XML1_NON_WS_SET, ALLOW_EXTRA_TEXT, PERSON_STANDARD);
        XML1_STRICT.testOk(PERSON_ODD_CHARS_XML1_OK_SET, STRICT_READ_OPTIONS, PERSON_ODD_CHARS);
        XML1_STRICT.testFail(PERSON_STANDARD_XML1_NON_WS_SET, STRICT_READ_OPTIONS, XmlTextAndChildException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_XML1_BAD_SYNTAX_SET, STRICT_READ_OPTIONS, XmlSyntaxException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_XML1_MISSING_NODE_SET, STRICT_READ_OPTIONS, MissingNodeException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_XML1_WRONG_NODE_SET, STRICT_READ_OPTIONS, WrongNodeException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_NS1_DEFAULT_OK, STRICT_READ_OPTIONS, WrongNodeException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_XML1_SURPLUS_NODE_SET, STRICT_READ_OPTIONS, SurplusNodeException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_XML1_WRONG_NODE_SET, STRICT_READ_OPTIONS, WrongNodeException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_NS1_OK, STRICT_READ_OPTIONS, WrongNodeException.class);
        XML1_STRICT.testFail(PERSON_STANDARD_XML1_CHILD_INSTEAD_OF_TEXT, STRICT_READ_OPTIONS, XmlNotOnlyTextException.class);
        XML1_STRICT.testOk(PERSON_STANDARD_XML1_NON_WS_SET, ALLOW_EXTRA_TEXT, PERSON_STANDARD);
        XML1_STRICT_WIDTH_FIRST.testOk(PERSON_STANDARD_XML1_OK, ALLOW_UNORDERED_ACCESS, PERSON_STANDARD);
        XML1_STRICT_WIDTH_FIRST.testFail(PERSON_STANDARD_XML1_OK, STRICT_READ_OPTIONS, UnorderedRetrievalException.class);
        XML1_USE_DEFAULTS.testOk(PERSON_STANDARD_XML1_MISSING_FIELD_SET, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML1_USE_DEFAULTS.testOk(PERSON_STANDARD_XML1_OK_SET, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML1_ALLOW_NULLS.testOk(PERSON_STANDARD_XML1_MISSING_FIRST_NAME, STRICT_READ_OPTIONS, PERSON_NO_FIRST_NAME);
        XML1_ALLOW_NULLS.testOk(PERSON_STANDARD_XML1_MISSING_SURNAME, STRICT_READ_OPTIONS, PERSON_NO_SURNAME);
        XML1_ALLOW_NULLS.testOk(PERSON_STANDARD_XML1_MISSING_OCCUPATION, STRICT_READ_OPTIONS, PERSON_NO_OCCUPATION);
        XML1_ALLOW_NULLS.testOk(PERSON_STANDARD_XML1_MISSING_NAME_FIELDS, STRICT_READ_OPTIONS, PERSON_NO_NAME);
        XML1_ALLOW_NULLS.testOk(PERSON_STANDARD_XML1_MISSING_FIELDS, STRICT_READ_OPTIONS, PERSON_NO_CONTENTS);
        XML_NS1_STRICT.testOk(PERSON_STANDARD_NS1_OK, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML_NS1_STRICT.testOk(PERSON_STANDARD_NS1_DEFAULT_OK, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML_NS1_STRICT.testOk(PERSON_STANDARD_NS1_PREFIX_TRICKS_OK, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML_NS1_STRICT.testFail(PERSON_STANDARD_NS2_OK, STRICT_READ_OPTIONS, WrongNodeException.class);
        XML_NS1_STRICT.testFail(PERSON_STANDARD_XML1_OK_SET, STRICT_READ_OPTIONS, WrongNodeException.class);
        XML_NS1_STRICT_NODE_ID.testOk(PERSON_STANDARD_NS1_OK, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML_ANY_NS.testOk(PERSON_STANDARD_XML1_OK, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML_ANY_NS.testOk(PERSON_STANDARD_NS1_OK, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML_ANY_NS.testFail(PERSON_BAD_NS_REF, STRICT_READ_OPTIONS, XmlSyntaxException.class);
        XML_ANY_NS.testFail(PERSON_2_NS_1_PREFIX_REF, STRICT_READ_OPTIONS, XmlSyntaxException.class);
        XML_ATTR1_STRICT.testOk(PERSON_STANDARD_ATTR1_OK_SET, STRICT_READ_OPTIONS, PERSON_STANDARD);
        XML_ATTR1_STRICT.testOk(PERSON_STANDARD_ATTR1_ODD_CHARS_SET, STRICT_READ_OPTIONS, PERSON_ODD_CHARS);
        XML_TEXT1_IGNORE_SEPARATORS.testOk(PERSON_STANDARD_TEXT1_OK, ALLOW_EXTRA_TEXT, PERSON_STANDARD);
        XML_TEXT1_IGNORE_SEPARATORS.testOk(PERSON_EMPTY_TEXT1, ALLOW_EXTRA_TEXT, PERSON_EMPTY_CONTENTS);
        XML_TEXT1_STRICT.testOk(PERSON_STANDARD_TEXT1_OK, ALLOW_EXTRA_TEXT, PERSON_STANDARD);
        XML_TEXT1_STRICT.testOk(PERSON_EMPTY_TEXT1, ALLOW_EXTRA_TEXT, PERSON_EMPTY_CONTENTS);
        XML_ANY_ORDER_STRICT.testOk(PERSON_STANDARD_XML1_OK, ALLOW_UNORDERED_ACCESS, PERSON_STANDARD);
        XML_ANY_ORDER_STRICT.testOk(PERSON_STANDARD_REVERSE, ALLOW_UNORDERED_ACCESS, PERSON_STANDARD);
        XML_ANY_ORDER_STRICT.testFail(PERSON_STANDARD_XML1_OK, STRICT_READ_OPTIONS, UnorderedRetrievalException.class);
        XML_ANY_ORDER_STRICT.testFail(PERSON_STANDARD_XML1_SURPLUS_NODE_SET, ALLOW_UNORDERED_ACCESS, SurplusNodeException.class);
        XML_ANY_ORDER_STRICT.testFail(PERSON_STANDARD_REVERSE_SURPLUS_NODE_SET, ALLOW_UNORDERED_ACCESS, SurplusNodeException.class);
        XML_ANY_ORDER_STRICT.testFail(PERSON_STANDARD_XML1_MISSING_NODE_SET, ALLOW_UNORDERED_ACCESS, MissingNodeException.class);
        XML_ANY_ORDER_STRICT.testFail(PERSON_STANDARD_XML1_WRONG_NODE_SET, ALLOW_UNORDERED_ACCESS, MissingNodeException.class);
        XML_ANY_ORDER_STRICT.testFail(PERSON_STANDARD_DUPLICATE_NODE, ALLOW_UNORDERED_ACCESS, DuplicateNodeException.class);
        XML_ANY_ORDER_WITH_DEFAULTS.testOk(PERSON_STANDARD_XML1_MISSING_FIELDS, ALLOW_UNORDERED_ACCESS, PERSON_STANDARD);
        XML_SEMI_ORDERED_STRICT.testOk(PERSON_STANDARD_XML1_OK, ALLOW_UNORDERED_ACCESS, PERSON_STANDARD);
        XML_SEMI_ORDERED_STRICT.testOk(PERSON_SEMI_REVERSE, ALLOW_UNORDERED_ACCESS, PERSON_STANDARD);
        XML_SEMI_ORDERED_STRICT.testFail(PERSON_STANDARD_XML1_OK, STRICT_READ_OPTIONS, UnorderedRetrievalException.class);
        announceTestClassPass();
    }
    /*
     * 
     */
    
    public static void
    staticTest()
    {
        XmlReaderSuiteNested t = new XmlReaderSuiteNested();
        t.test();
    }
    
    public static void
    main(String[] args)
    {
        staticTest();
    }
}
